.model tiny
.186
.code
org 100h

start:      mov   ax, 13h           ; set video mode 13h (320x200x256)
            int   10h

            mov   ds, ax            ; seed random number with clock timer,
            xchg  bx, ax            ; also set bx to 13h which is a small
            mov   ax, ds:[033Ch]    ; enough value that the loop below will
                                    ; cover almost the whole screen
            mov   dx, cs            ; ds = segment of the working buffer
            add   dh, 10h           ; here we use the 64K segment immediately
            mov   ds, dx            ; following the code segment

            push  0A000h            ; es = video memory (at 0A000h)
            pop   es
                                    ; random number generation loop:
rand:       imul  ax, 9421          ;  generate random number using linear
            inc   ax                ;  congruential formula 9421*n+1
            xor   [bx], ah          ;  xor into this byte in the buffer, then
            inc   bx                ;  loop back to do next byte
            jnz   rand              ;  bits other than the low bit are set,
                                    ;  but this will only mess up the screen
                                    ;  for the first few frames

            lea   di, [si-322]      ; di = si - 322, it doesn't matter what
                                    ; si contains because this is a byte loop

; The counting loop will not run 3 times on the first pixel, but it will run
; correctly after that (see below).

life:       xor   al, al            ; pixel counting loop:
count:      add   al, [di+bx]       ;  add in the top pixel in this column,
            add   al, [si+bx-2]     ;  then the middle one, and finally
            add   al, [si+bx+318]   ;  the bottom pixel; then move to the
            dec   bx                ;  next column and loop until bx is zero
            jnz   count

            sub   al, [si]          ; subtract out center pixel
            or    al, [si]          ; al = 3 if pixel will be on
                                    ; if it was 3, it stays 3 (3=birth)
                                    ; if it was 2, is 3 if [si]=1 (2=stay)
            aaa                     ; mask off high bits

            movsb                   ; write pixel to screen

            mov   bl, 3             ; bl = 3 for 3 iterations of count loop
            cmp   al, bl            ; compare al to 3
            jne   cdone             ; if al = 3, then set new pixel = 1
            or    byte ptr [si-1], 80h

cdone:      shr   byte ptr [di], 7  ; clear previous pixel
            loop  life              ; jump fails once each frame...

            mov   ah, 1             ; check for keypress
            int   16h
            jz    life              ; loop while no key has been pressed

            xchg  ax, bx            ; restore text mode (bx = 3)
            int   10h
            ret                     ; return to dos

end start
